"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const matchers_1 = require("./matchers");
class MockObject {
    static fromPrototype(prototype) {
        function isFunction(property) {
            return typeof property === "function";
        }
        let functions = Object.getOwnPropertyNames(prototype)
            .filter((p) => (isFunction(prototype[p])))
            .filter((p) => (p !== "constructor"));
        return this.fromFunctionNames(functions);
    }
    static fromFunctionNames(functionNames) {
        let mock = {};
        functionNames.forEach((fn) => mock[fn] = this._createMockFunction());
        return mock;
    }
    static _createMockFunction() {
        let expectedCalls = new Map();
        let fn = jest.fn();
        fn.mockImplementation(this._handleCall.bind(null, expectedCalls, fn.mock.calls));
        fn["mockReturnValueWithArgs"] = this._mockReturnValueWithArgs.bind(null, expectedCalls);
        return fn;
    }
    static _handleCall(expectedCalls, calls) {
        let lastCall = calls[calls.length - 1];
        let result;
        expectedCalls.forEach((value, key, map) => {
            if (lastCall.length === key.length) {
                let matches = true;
                for (let i = 0; i < key.length; i++) {
                    let matcher = key[i];
                    if (!(matcher(lastCall[i]))) {
                        matches = false;
                        break;
                    }
                }
                if (matches) {
                    result = value;
                    return;
                }
            }
        });
        if (result) {
            return result;
        }
    }
    static _mockReturnValueWithArgs(expectedCalls, args, value) {
        function isFunction(a) {
            return typeof a === "function";
        }
        let keys = [];
        args.forEach((a) => {
            if (!isFunction(a)) {
                keys.push(matchers_1.eq(a));
            }
            else {
                keys.push(a);
            }
        });
        expectedCalls.set(keys, value);
    }
}
exports.MockObject = MockObject;
